# Copyright 2020-2024 Flavio Garcia
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64


def jose_b64(data):
    """ Encodes data with JOSE/JWS base 64 encoding.
    """
    return base64.urlsafe_b64encode(data).decode('ascii').replace("=", "")


def number_to_bytes(number, byteorder="big"):
    return number.to_bytes(number.bit_length() // 8 + 1, byteorder=byteorder)


def to_jwk(key, kid=None):
    """ Creates a new request header for the specified account key.
    """
    from cryptography.hazmat.primitives.asymmetric.ec import (
        EllipticCurvePrivateKey
    )
    from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey

    if isinstance(key, EllipticCurvePrivateKey):
        numbers = key.public_key().public_numbers()
        key = {
            'alg': f"ES{key.key_size}",
            'kty': "EC",
            'crv': f"P-{key.key_size}",
            'x': jose_b64(number_to_bytes(numbers.x)),
            'y': jose_b64(number_to_bytes(numbers.y)),
        }
        if kid is not None:
            key['kid'] = kid
        return key

    if isinstance(key, RSAPrivateKey):
        numbers = key.public_key().public_numbers()
        # if n[0] == 0:  # for strict JWK
        #     n = n[1:]
        key = {
            'alg': "RS256",
            'kty': "RSA",
            'e': jose_b64(number_to_bytes(numbers.e)),
            'n': jose_b64(number_to_bytes(numbers.n)),
        }
        if kid is not None:
            key['kid'] = kid
        return key

    raise NotImplementedError("Key is neigher RSA or EC.")
